unit At_Class;
{*********************************************}
{                                             }
{   THERMOCHEMICAL CLASSES (Atoms)            }
{                                             }
{   (C) 2000 Belov G.V., IVTAN, Thermocenter  }
{                                             }
{   Please send bugs, comments and            }
{   suggestions to:                            }
{                                             }
{   gbelov@yandex.ru                           }
{                                             }
{*********************************************}
interface
uses SysUtils, AtConst;

const MaxNumberOfElements = 109;
Type TElement = class
  private
    FMendeleev_No :     byte;  // number of element in periodic table
    FTPX_no :           byte;  // thermochemical number of element
    FFormula :          ST2;   // symbol of element
    FStandard_Formula : ST20;      // formula of element in standard state
    FName              : ST20;      // name of the element
    FTmelt :            Double;    // melting temperature (K)
    FPmelt :            Double;    // pressure for melting point (atm)
    FTboil :            Double;    // boiling temperature (K) at 1 atm
    FDensity :          Double;    // density
    FDensUnit :         ST20;      // density units
    FTdens :            Double;    // temperature for density (K)
    FPot_Ion :          Double;    // ionization potential (eV)
    FHmelt :            Double;    // heat of melting (kJ/mol)
    FHvap :             Double;    // heat of vaporization (kJ/mol)
    FAt_FS :            byte;      // number of atoms in formula for standard state
    FAtomic_Mass :      Double;    // atomic mass
    FSnuclR :           Double;    // Snucl/R (nuclear spin component divided by R0)
    FMend_Group :       ST2;       // group in periodic table
  protected
    procedure SetMendeleev_No(S : byte); virtual;
    procedure SetTPX_No(S : byte); virtual;
    procedure SetFormula(S : ST2); virtual;
    procedure SetStandard_Formula(S : ST20); virtual;
    procedure SetName(S : ST20); virtual;
    procedure SetTmelt(S: Double); virtual;
    procedure SetPmelt(S: Double); virtual;
    procedure SetTboil(S: Double); virtual;
    procedure SetDensity(S: Double); virtual;
    procedure SetDensUnit(S: ST20); virtual;
    procedure SetTdens(S: Double); virtual;
    procedure SetPot_Ion(S: Double); virtual;
    procedure SetHmelt(S: Double); virtual;
    procedure SetHvap(S: Double); virtual;
    procedure SetAt_FS(S: byte); virtual;
    procedure SetAtomic_Mass(S : Double); virtual;
    procedure SetSnuclR(S: Double); virtual;
    procedure SetMend_Group(S: ST2); virtual;
  public
    procedure SetAllData(MND_No,TPX_No, At_FS : byte; Form: ST2;
                       StandFml : ST20; Name : ST20;
                       Tmelt, Pmelt, Tboil, Density : Double;
                       UnitDens : ST20;
                       Tdens, Pot_Ion, Hmelt, Hvap, At_Mass, SnuclR : Double;
                       Mend_Group : ST2); virtual;
    property Mendeleev_No: byte read FMendeleev_No write SetMendeleev_No;
    property TPX_No: byte read FTPX_No write SetTPX_No;
    property Formula: ST2 read FFormula write SetFormula;
    property Standard_Formula: ST20 read FStandard_Formula write SetStandard_Formula;
    property Name: ST20 read FName write SetName;
    property Tmelt : Double read FTmelt write SetTmelt;
    property Pmelt : Double read FPmelt write SetPmelt;
    property Tboil : Double read FTboil write SetTboil;
    property Density : Double read FDensity write SetDensity;
    property DensUnit : ST20 read FDensUnit write SetDensUnit;
    property TDense : Double read FTdens write SetTdens;
    property Pot_Ion : Double read FPot_Ion write SetPot_Ion;
    property Hmelt : Double read FHmelt write SetHmelt;
    property Hvap : Double read FHvap write SetHvap;
    property At_FS: byte read FAt_FS write SetAt_FS;
    property Atomic_Mass: Double read FAtomic_Mass write SetAtomic_Mass;
    property SnuclR : Double read FSnuclR write SetSnuclR;
    property Mend_Group : ST2 read FMend_Group write SetMend_Group;
  end;

  TElements = array[1..MaxNumberOfElements] of TElement;

  TMendTab = class
  private
    FElements : TElements;        // array of information on atoms (of TAtom type)
    FElementsNumber : integer;    // real number of atoms in the table
  protected

  public
    constructor Create; virtual;
    destructor Destroy; override;
    function ElementByMendNo(pIndex: integer): TElement; virtual;
    function ElementByTPXNo(pIndex: integer): TElement; virtual;
    function ElementByName(pIndex: ST2): TElement; virtual;
    function ElementByOrder(pIndex: integer): TElement; virtual;
  end;

var MendTab : TMendTab;
    Element : TElement;
implementation

procedure TElement.SetAllData(MND_No,TPX_No, At_FS : byte; Form: ST2;
                       StandFml : ST20; Name : ST20;
                       Tmelt, Pmelt, Tboil, Density : Double;
                       UnitDens : ST20;
                       Tdens, Pot_Ion, Hmelt, Hvap, At_Mass, SnuclR : Double;
                       Mend_Group : ST2);
begin
  FMendeleev_No := MND_No;
  FTPX_No := TPX_No;
  FFormula := Form;
  FStandard_Formula := StandFml;
  FName := Name;
  FTmelt := Tmelt;
  FPmelt := Pmelt;
  FTboil := Tboil;
  FDensity := Density;
  FDensUnit := UnitDens;
  FTdens := Tdens;
  FPot_Ion := Pot_Ion;
  FHmelt := Hmelt;
  FHvap := Hvap;
  FAt_FS := At_FS;
  FAtomic_Mass := At_Mass;
  FSnuclR := SnuclR;
  FMend_Group := Mend_Group;
end;

procedure TElement.SetMendeleev_No(S : byte);
begin
  FMendeleev_No := S;
end;

procedure TElement.SetTPX_No(S : byte);
begin
  FTPX_No := S;
end;

procedure TElement.SetFormula(S : ST2);
begin
  FFormula := S;
end;

procedure TElement.SetStandard_Formula(S : ST20);
begin
  FStandard_Formula := S;
end;

procedure TElement.SetName(S : ST20);
begin
  FName := S;
end;

procedure TElement.SetTmelt(S : Double);
begin
  FTmelt := S;
end;

procedure TElement.SetPmelt(S : Double);
begin
  FPmelt := S;
end;

procedure TElement.SetTBoil(S : Double);
begin
  FTboil := S;
end;

procedure TElement.SetDensity(S : Double);
begin
  FDensity := S;
end;

procedure TElement.SetDensUnit(S : ST20);
begin
  FDensUnit := S;
end;

procedure TElement.SetTdens(S : Double);
begin
  FTdens := S;
end;

procedure TElement.SetPot_Ion(S : Double);
begin
  FPot_Ion := S;
end;

procedure TElement.SetHmelt(S : Double);
begin
  FHmelt := S;
end;

procedure TElement.SetHvap(S : Double);
begin
  FHvap := S;
end;

procedure TElement.SetAt_FS(S : byte);
begin
  FAt_FS := S;
end;

procedure TElement.SetAtomic_Mass(S : Double);
begin
  FAtomic_Mass := S;
end;

procedure TElement.SetSnuclR(S : Double);
begin
  FSnuclR := S;
end;

procedure TElement.SetMend_Group(S : ST2);
begin
  FMend_Group := S;
end;

constructor TMendTab.Create;
var Element : TElement;
    i : integer;
    MND_No,TPX_No,At_FS : byte;
    Form: ST2;
    StandFml, Name : ST20;
    Tmelt, Pmelt, Tboil, Density : Double;
    UnitDens : ST20;
    Tdens, Pot_Ion, Hmelt, Hvap, At_Mass, SnuclR : Double;
    Mend_Group : ST2;
begin
  FElementsNumber := 0;
    begin
      for i := 1 to High(AtomicData) do
        begin
        MND_No := AtomicData[i].Mendeleev_No;
        TPX_No := AtomicData[i].TPX_No;
        At_FS  := AtomicData[i].At_FS;
        Form := AtomicData[i].Formula;
        StandFml := AtomicData[i].Standard_Formula;
        Name := AtomicData[i].Name;
        Tmelt := AtomicData[i].Tmelt;
        Pmelt := AtomicData[i].Pmelt;
        Tboil := AtomicData[i].Tboil;
        Density := AtomicData[i].Density;
        UnitDens := AtomicData[i].DensUnit;
        Tdens := AtomicData[i].Tdens;
        Pot_Ion := AtomicData[i].Pot_Ion;
        Hmelt := AtomicData[i].Hmelt;
        Hvap := AtomicData[i].Hvap;
        At_Mass := AtomicData[i].Atomic_MAss;
        SnuclR := AtomicData[i].SnuclR;
        Mend_Group := AtomicData[i].Mend_Group;
        Element := TElement.Create;
        Element.SetAllData(MND_No,TPX_No, At_FS, Form, StandFml, Name,
                             Tmelt, Pmelt, Tboil, Density,
                             UnitDens,
                             Tdens, Pot_Ion, Hmelt, Hvap, At_Mass, SnuclR,
                             Mend_Group);
        inc(FElementsNumber);
        if FElementsNumber > MaxNumberOfElements then
          Raise Exception.Create('Too many elements: '+IntToStr(MaxNumberOfElements));
        FElements[FElementsNumber] := Element;
        end;
        Exit;
    end;
end;

destructor TMendTab.Destroy;
var i : integer;
begin
  for i := 1 to FElementsNumber do
    FElements[i].Free;
  inherited Destroy;
end;

function TMendTab.ElementByMendNo(pIndex: integer) : TElement;
var i : integer;
begin
  if (pIndex < 0) or (pIndex > FElementsNumber) then
    Raise Exception.Create('Invalid number: '+IntToStr(pIndex));
  for i := 1 to FElementsNumber do
    if FElements[i].Mendeleev_No = pIndex then
      begin
      Result := FElements[i];
      Exit;
      end;
    Raise Exception.Create('Element number '+IntToStr(pIndex)+' not found');
end;

function TMendTab.ElementByTPXNo(pIndex: integer) : TElement;
var i : integer;
begin
  if (pIndex < 0) or (pIndex > FElementsNumber) then
    Raise Exception.Create('Invalid TPX number: '+IntToStr(pIndex));
  for i := 1 to FElementsNumber do
    if FElements[i].TPX_No = pIndex then
      begin
      Result := FElements[i];
      Exit;
      end;
    Raise Exception.Create('Element with TPX number '+IntToStr(pIndex)+' not found');
end;

function TMendTab.ElementByName(pIndex: ST2) : TElement;
var i : integer;
begin
  for i := 1 to FElementsNumber do
    if FElements[i].Formula = pIndex then
      begin
      Result := FElements[i];
      Exit;
      end;
    Raise Exception.Create('Element '+pIndex+' not found');
end;

function TMendTab.ElementByOrder(pIndex: integer) : TElement;
begin
  if (pIndex < 1) or (pIndex > FElementsNumber) then
    Raise Exception.Create('Invalid number: '+IntToStr(pIndex));
  Result := FElements[pIndex];
end;

end.